@@ -1,3 +1,6 @@
+// SPDX-License-Identifier: Apache-2.0
+// Copyright Contributors to the OpenTimelineIO project
+
 #pragma once
 
 #include "opentime/timeRange.h"
@@ -27,11 +30,16 @@ public:
         optional<TimeRange> const&  source_range = nullopt,
         AnyDictionary const&        metadata     = AnyDictionary(),
         std::vector<Effect*> const& effects      = std::vector<Effect*>(),
-        std::vector<Marker*> const& markers      = std::vector<Marker*>());
+        std::vector<Marker*> const& markers      = std::vector<Marker*>(),
+        bool                        enabled      = true);
 
     virtual bool visible() const;
     virtual bool overlapping() const;
 
+    bool enabled() const { return _enabled; };
+
+    void set_enabled(bool enabled) { _enabled = enabled; }
+
     optional<TimeRange> source_range() const noexcept { return _source_range; }
 
     void set_source_range(optional<TimeRange> const& source_range)
@@ -90,6 +98,7 @@ private:
     optional<TimeRange>           _source_range;
     std::vector<Retainer<Effect>> _effects;
     std::vector<Retainer<Marker>> _markers;
+    bool                          _enabled;
 };
 
 }} // namespace opentimelineio::OPENTIMELINEIO_VERSION
