@@ -1,3 +1,6 @@
+// SPDX-License-Identifier: Apache-2.0
+// Copyright Contributors to the OpenTimelineIO project
+
 #pragma once
 
 #include "opentimelineio/composition.h"
@@ -41,15 +44,18 @@ public:
     void set_kind(std::string const& kind) { _kind = kind; }
 
     virtual TimeRange range_of_child_at_index(
-        int index, ErrorStatus* error_status = nullptr) const;
+        int          index,
+        ErrorStatus* error_status = nullptr) const;
     virtual TimeRange trimmed_range_of_child_at_index(
-        int index, ErrorStatus* error_status = nullptr) const;
+        int          index,
+        ErrorStatus* error_status = nullptr) const;
     virtual TimeRange
     available_range(ErrorStatus* error_status = nullptr) const;
 
     virtual std::pair<optional<RationalTime>, optional<RationalTime>>
     handles_of_child(
-        Composable const* child, ErrorStatus* error_status = nullptr) const;
+        Composable const* child,
+        ErrorStatus*      error_status = nullptr) const;
 
     std::pair<Retainer<Composable>, Retainer<Composable>> neighbors_of(
         Composable const* item,
@@ -59,6 +65,9 @@ public:
     virtual std::map<Composable*, TimeRange>
     range_of_all_children(ErrorStatus* error_status = nullptr) const;
 
+    optional<Imath::Box2d>
+    available_image_bounds(ErrorStatus* error_status) const;
+
     // Return a vector of clips.
     //
     // An optional search_range may be provided to limit the search.
