@@ -1,26 +1,5 @@
-#
+# SPDX-License-Identifier: Apache-2.0
 # Copyright Contributors to the OpenTimelineIO project
-#
-# Licensed under the Apache License, Version 2.0 (the "Apache License")
-# with the following modification; you may not use this file except in
-# compliance with the Apache License and the following modification to it:
-# Section 6. Trademarks. is deleted and replaced with:
-#
-# 6. Trademarks. This License does not grant permission to use the trade
-#    names, trademarks, service marks, or product names of the Licensor
-#    and its affiliates, except as required to comply with Section 4(c) of
-#    the License and to reproduce the content of the NOTICE file.
-#
-# You may obtain a copy of the Apache License at
-#
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the Apache License with the above modification is
-# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
-# KIND, either express or implied. See the Apache License for the specific
-# language governing permissions and limitations under the Apache License.
-#
 
 """OpenTimelineIO CMX 3600 EDL Adapter"""
 
@@ -36,7 +15,6 @@
 import copy
 import os
 import re
-import math
 import collections
 
 from .. import (
@@ -109,10 +87,17 @@ class EDLParser(object):
 
         # TODO: Sort the tracks V, then A1,A2,etc.
 
-    def add_clip(self, line, comments, rate=24):
+    def add_clip(self, line, comments, rate=24, transition_line=None):
         comment_handler = CommentHandler(comments)
-        clip_handler = ClipHandler(line, comment_handler.handled, rate=rate)
+        clip_handler = ClipHandler(
+            line,
+            comment_handler.handled,
+            rate=rate,
+            transition_line=transition_line
+        )
         clip = clip_handler.clip
+        transition = clip_handler.transition
+        # Add unhandled comments as general comments to meta data.
         if comment_handler.unhandled:
             clip.metadata.setdefault("cmx_3600", {})
             clip.metadata['cmx_3600'].setdefault("comments", [])
@@ -129,15 +114,6 @@ class EDLParser(object):
             clip.metadata.setdefault("cmx_3600", {})
             clip.metadata['cmx_3600']['reel'] = clip_handler.reel
 
-        # each edit point between two clips is a transition. the default is a
-        # cut in the edl format the transition codes are for the transition
-        # into the clip
-        self.add_transition(
-            clip_handler,
-            clip_handler.transition_type,
-            clip_handler.transition_data
-        )
-
         edl_rate = clip_handler.edl_rate
         record_in = opentime.from_timecode(
             clip_handler.record_tc_in,
@@ -195,8 +171,11 @@ class EDLParser(object):
         # Add clip instances to the tracks
         tracks = self.tracks_for_channel(clip_handler.channel_code)
         for track in tracks:
+            track_transition = transition
             if len(tracks) > 1:
                 track_clip = copy.deepcopy(clip)
+                if transition:
+                    track_transition = copy.deepcopy(transition)
             else:
                 track_clip = clip
 
@@ -234,6 +213,12 @@ class EDLParser(object):
                 track.append(gap)
                 _extend_source_range_duration(track, gap.duration())
 
+            if track_transition:
+                if len(track) < 1:
+                    raise EDLParseError(
+                        "Transitions can't be at the very beginning of a track"
+                    )
+                track.append(track_transition)
             track.append(track_clip)
             _extend_source_range_duration(track, track_clip.duration())
 
@@ -264,12 +249,6 @@ class EDLParser(object):
         # Return a list of actual tracks
         return [self.tracks_by_name[c] for c in track_names]
 
-    def add_transition(self, clip_handler, transition, data):
-        if transition not in ['C']:
-            md = clip_handler.clip.metadata.setdefault("cmx_3600", {})
-            md["transition"] = transition
-            md["transition_duration"] = float(data)
-
     def parse_edl(self, edl_string, rate=24):
         # edl 'events' can be comprised of an indeterminate amount of lines
         # we are to translating 'events' to a single clip and transition
@@ -322,7 +301,7 @@ class EDLParser(object):
 
                 line_1 = edl_lines.pop(0)
                 line_2 = edl_lines.pop(0)
-
+                # TODO: check if transitions can happen in this case
                 comments = []
                 while edl_lines:
                     if re.match(r'^\D', edl_lines[0]):
@@ -331,24 +310,45 @@ class EDLParser(object):
                         break
                 self.add_clip(line_1, comments, rate=rate)
                 self.add_clip(line_2, comments, rate=rate)
-
+            # Check if the first character in the line is a digit
             elif line[0].isdigit():
+                transition_line = None
                 # all 'events' start_time with an edit decision. this is
                 # denoted by the line beginning with a padded integer 000-999
                 comments = []
+                event_id = int(re.match(r'^\d+', line).group(0))
                 while edl_lines:
-                    # any non-numbered lines after an edit decision should be
-                    # treated as 'comments'
-                    # comments are string tags used by the reader to get extra
+                    # Any non-numbered lines after an edit decision should be
+                    # treated as 'comments'.
+                    # Comments are string tags used by the reader to get extra
                     # information not able to be found in the restricted edl
-                    # format
-                    if re.match(r'^\D', edl_lines[0]):
+                    # format.
+                    # If the current event id is repeated it means that there is
+                    # a transition between the current event and the preceding
+                    # one. We collect it and process it when adding the clip.
+                    m = re.match(r'^\d+', edl_lines[0])
+                    if not m:
                         comments.append(edl_lines.pop(0))
                     else:
-                        break
-
-                self.add_clip(line, comments, rate=rate)
+                        if int(m.group(0)) == event_id:
+                            # It is not possible to have multiple transitions
+                            # for the same event.
+                            if transition_line:
+                                raise EDLParseError(
+                                    'Invalid transition %s' % edl_lines[0]
+                                )
+                            # Same event id, this is a transition
+                            transition_line = edl_lines.pop(0)
+                        else:
+                            # New event, stop collecting comments and transitions
+                            break
 
+                self.add_clip(
+                    line,
+                    comments,
+                    rate=rate,
+                    transition_line=transition_line
+                )
             else:
                 raise EDLParseError('Unknown event type')
 
@@ -365,20 +365,26 @@ class ClipHandler(object):
         r'.*\.(?P<range>\[(?P<start>[0-9]+)-(?P<end>[0-9]+)\])\.\w+$'
     )
 
-    def __init__(self, line, comment_data, rate=24):
+    def __init__(self, line, comment_data, rate=24, transition_line=None):
         self.clip_num = None
         self.reel = None
         self.channel_code = None
         self.edl_rate = rate
         self.transition_id = None
         self.transition_data = None
+        self.transition_type = None
         self.source_tc_in = None
         self.source_tc_out = None
         self.record_tc_in = None
         self.record_tc_out = None
-
+        self.clip = None
+        self.transition = None
         self.parse(line)
+        if transition_line:
+            self.parse(transition_line)
         self.clip = self.make_clip(comment_data)
+        if transition_line:
+            self.transition = self.make_transition(comment_data)
 
     def is_image_sequence(self, comment_data):
         return self.image_sequence_pattern.search(
@@ -413,7 +419,7 @@ class ClipHandler(object):
 
         # BLACK/BL and BARS are called out as "Special Source Identifiers" in
         # the documents referenced here:
-        # https://github.com/PixarAnimationStudios/OpenTimelineIO#cmx3600-edl
+        # https://github.com/AcademySoftwareFoundation/OpenTimelineIO#cmx3600-edl
         if self.reel in ['BL', 'BLACK']:
             clip.media_reference = schema.GeneratorReference()
             # TODO: Replace with enum, once one exists
@@ -497,11 +503,8 @@ class ClipHandler(object):
                 }
             }
 
-        # In transitions, some of the source clip metadata might fall in the
-        # transition clip event
+        # Get the clip name from "TO CLIP NAME" if present
         if 'dest_clip_name' in comment_data:
-            previous_meta = clip.metadata.setdefault('previous_metadata', {})
-            previous_meta['source_clip_name'] = clip.name
             clip.name = comment_data['dest_clip_name']
 
         if 'locators' in comment_data:
@@ -571,7 +574,7 @@ class ClipHandler(object):
             # denotes frame count
             # i haven't figured out how the key transitions (K, KB, KO) work
             (
-                self.clip_num,
+                self.transition_id,
                 self.reel,
                 self.channel_code,
                 self.transition_type,
@@ -583,19 +586,26 @@ class ClipHandler(object):
             ) = fields
 
         elif field_count == 8:
+            edit_type = None
             # no transition data
             # this is for basic cuts
             (
                 self.clip_num,
                 self.reel,
                 self.channel_code,
-                self.transition_type,
+                edit_type,
                 self.source_tc_in,
                 self.source_tc_out,
                 self.record_tc_in,
                 self.record_tc_out
             ) = fields
-
+            # Double check it is a cut
+            if edit_type not in ["C"]:
+                raise EDLParseError(
+                    'incorrect edit type {} in form statement: {}'.format(
+                        edit_type, line,
+                    )
+                )
         else:
             raise EDLParseError(
                 'incorrect number of fields [{0}] in form statement: {1}'
@@ -621,6 +631,75 @@ class ClipHandler(object):
                     )
                 )
 
+    def make_transition(self, comment_data):
+        # Do some sanity check
+        if not self.clip:
+            raise RuntimeError("Transitions can't be handled without a clip")
+        if self.transition_id != self.clip_num:
+            raise EDLParseError(
+                'transition and event id mismatch: {} vs {}'.format(
+                    self.transaction_id, self.clip_num,
+                )
+            )
+        if re.match(r'W(\d{3})', self.transition_type):
+            otio_transition_type = "SMPTE_Wipe"
+        elif self.transition_type in ['D']:
+            otio_transition_type = schema.TransitionTypes.SMPTE_Dissolve
+        else:
+            raise EDLParseError(
+                "Transition type '{}' not supported by the CMX EDL reader "
+                "currently.".format(self.transition_type)
+            )
+        # TODO: support delayed transition like described here:
+        # https://xmil.biz/EDL-X/CMX3600.pdf
+        transition_duration = opentime.RationalTime(
+            float(self.transition_data),
+            self.clip.source_range.duration.rate
+        )
+        # Note: Transitions in EDLs are unconventionally represented.
+        #
+        # Where a transition might normally be visualized like:
+        #            |---57.0 Trans 43.0----|
+        # |------Clip1 102.0------|----------Clip2 143.0----------|Clip3 24.0|
+        #
+        # In an EDL it can be thought of more like this:
+        #            |---0.0 Trans 100.0----|
+        # |Clip1 45.0|----------------Clip2 200.0-----------------|Clip3 24.0|
+        #
+        # So the transition starts at the beginning of the clip with `duration`
+        # frames from the previous clip.
+
+        # Give the transition a detailed name if we can
+        transition_name = '{} to {}'.format(
+            otio_transition_type,
+            self.clip.name,
+        )
+        if 'dest_clip_name' in comment_data:
+            if 'clip_name' in comment_data:
+                transition_name = '{} from {} to {}'.format(
+                    otio_transition_type,
+                    comment_data['clip_name'],
+                    comment_data['dest_clip_name'],
+                )
+
+        new_trx = schema.Transition(
+            name=transition_name,
+            # only supported type at the moment
+            transition_type=otio_transition_type,
+            metadata={
+                'cmx_3600': {
+                    'transition': self.transition_type,
+                    'transition_duration': transition_duration.value,
+                }
+            },
+        )
+        new_trx.in_offset = opentime.RationalTime(
+            0,
+            transition_duration.rate
+        )
+        new_trx.out_offset = transition_duration
+        return new_trx
+
 
 class CommentHandler(object):
     # this is the for that all comment 'id' tags take
@@ -672,218 +751,6 @@ class CommentHandler(object):
                 self.unhandled.append(stripped)
 
 
-def _get_next_clip(start_index, track):
-    """Get the next clip with a non-zero duration"""
-    # Iterate over the following clips and return the first "real" one
-    for clip in track[start_index + 1:]:
-        if clip.duration().value > 0:
-            return clip
-
-    return None
-
-
-def _expand_transitions(timeline):
-    """Convert clips with metadata/transition == 'D' into OTIO transitions."""
-
-    tracks = timeline.tracks
-    remove_list = []
-    replace_or_insert_list = []
-    append_list = []
-    for track in tracks:
-        # avid inserts an extra clip for the source
-        prev_prev = None
-        prev = None
-        for index, clip in enumerate(track):
-            transition_type = clip.metadata.get('cmx_3600', {}).get(
-                'transition',
-                'C'
-            )
-
-            if transition_type == 'C':
-                # nothing to do, continue to the next iteration of the loop
-                prev_prev = prev
-                prev = clip
-                continue
-
-            wipe_match = re.match(r'W(\d{3})', transition_type)
-            if wipe_match is not None:
-                otio_transition_type = "SMPTE_Wipe"
-            elif transition_type in ['D']:
-                otio_transition_type = schema.TransitionTypes.SMPTE_Dissolve
-            else:
-                raise EDLParseError(
-                    "Transition type '{}' not supported by the CMX EDL reader "
-                    "currently.".format(transition_type)
-                )
-
-            # Using transition data for duration (with clip duration as backup.)
-            # Link: https://ieeexplore.ieee.org/document/7291839
-            # Citation: "ST 258:2004 - SMPTE Standard - For Television - Transfer
-            #   of Edit Decision Lists," in ST 258:2004 , vol., no., pp.1-37,
-            #   6 April 2004, doi: 10.5594/SMPTE.ST258.2004.
-            if clip.metadata.get("cmx_3600", {}).get("transition_duration"):
-                transition_duration = opentime.RationalTime(
-                    clip.metadata["cmx_3600"]["transition_duration"],
-                    clip.duration().rate
-                )
-            else:
-                transition_duration = clip.duration()
-
-            # EDL doesn't have enough data to know where the cut point was, so
-            # this arbitrarily puts it in the middle of the transition
-            pre_cut = math.floor(transition_duration.value / 2)
-            post_cut = transition_duration.value - pre_cut
-            mid_tran_cut_pre_duration = opentime.RationalTime(
-                pre_cut,
-                transition_duration.rate
-            )
-            mid_tran_cut_post_duration = opentime.RationalTime(
-                post_cut,
-                transition_duration.rate
-            )
-
-            # Because transitions can have two event entries followed by
-            # comments, some of the previous clip's metadata might land in the
-            # transition clip
-            if prev:
-                if 'previous_metadata' in clip.metadata:
-                    prev_metadata = clip.metadata['previous_metadata']
-                    if 'source_clip_name' in prev_metadata:
-                        # Give the transition the event name and the
-                        # previous clip the appropriate name
-                        prev.name = prev_metadata['source_clip_name']
-
-            # expand the previous
-            expansion_clip = None
-            if prev and not prev_prev:
-                expansion_clip = prev
-            elif prev_prev:
-                # If the previous clip is continuous to this one, we can combine
-                if _transition_clips_continuous(prev_prev, prev):
-                    expansion_clip = prev_prev
-                    if prev:
-                        remove_list.append((track, prev))
-                else:
-                    expansion_clip = prev
-
-            _extend_source_range_duration(expansion_clip, mid_tran_cut_pre_duration)
-
-            # rebuild the clip as a transition
-            new_trx = schema.Transition(
-                name=clip.name,
-                # only supported type at the moment
-                transition_type=otio_transition_type,
-                metadata=clip.metadata,
-            )
-            new_trx.in_offset = mid_tran_cut_pre_duration
-            new_trx.out_offset = mid_tran_cut_post_duration
-
-            # expand the next_clip or contract this clip
-            keep_transition_clip = False
-            next_clip = _get_next_clip(index, track)
-            if next_clip:
-                if _transition_clips_continuous(clip, next_clip):
-                    sr = next_clip.source_range
-                    next_clip.source_range = opentime.TimeRange(
-                        sr.start_time - mid_tran_cut_post_duration,
-                        sr.duration + mid_tran_cut_post_duration,
-                    )
-                else:
-                    # The clip was only expressed in the transition, keep it,
-                    # though it needs the previous clip transition time removed
-                    keep_transition_clip = True
-
-                    sr = clip.source_range
-                    clip.source_range = opentime.TimeRange(
-                        sr.start_time + mid_tran_cut_pre_duration,
-                        sr.duration - mid_tran_cut_pre_duration,
-                    )
-            else:
-                fill = schema.Gap(
-                    source_range=opentime.TimeRange(
-                        duration=mid_tran_cut_post_duration,
-                        start_time=opentime.RationalTime(
-                            0,
-                            transition_duration.rate
-                        )
-                    )
-                )
-                append_list.append((track, fill))
-
-            #                   in     from  to
-            replace_or_insert_list.append((keep_transition_clip, track, clip, new_trx))
-
-            # Scrub some temporary metadata stashed on clips about their
-            # neighbors
-            if 'previous_metadata' in clip.metadata:
-                del(clip.metadata['previous_metadata'])
-
-            if 'previous_metadata' in new_trx.metadata:
-                del(new_trx.metadata['previous_metadata'])
-
-            prev = clip
-
-    for (insert, track, from_clip, to_transition) in replace_or_insert_list:
-        clip_index = track.index(from_clip)
-        if insert:
-            track.insert(clip_index, to_transition)
-        else:
-            track[clip_index] = to_transition
-
-    for (track, clip_to_remove) in list(set(remove_list)):
-        # if clip_to_remove in track:
-        track.remove(clip_to_remove)
-
-    for (track, clip) in append_list:
-        track.append(clip)
-
-    return timeline
-
-
-def _transition_clips_continuous(clip_a, clip_b):
-    """Tests if two clips are continuous. They are continuous if the following
-    conditions are met:
-        1. clip_a's source range ends on the last frame before clip_b's
-        2a. If clip_a's name matches clip_b's
-        - or -
-        2b. clip_a name matches metadata source_clip_name in clip_b
-        - or -
-        2c. Reel name matches
-        - or -
-        2d. Both clips are gaps
-
-
-    This is specific to how this adapter parses EDLs and is meant to be run only
-    within _expand_transitions.
-    """
-    clip_a_end = clip_a.source_range.end_time_exclusive()
-    if not clip_a_end == clip_b.source_range.start_time:
-        return False
-
-    if all(isinstance(clip, schema.Gap) for clip in (clip_a, clip_b)):
-        return True
-
-    # The time ranges are continuous, match the names
-    if (clip_a.name == clip_b.name):
-        return True
-
-    def reelname(clip):
-        return clip.metadata['cmx_3600']['reel']
-
-    try:
-        if reelname(clip_a) == reelname(clip_b):
-            return True
-    except KeyError:
-        pass
-
-    try:
-        return clip_a.name == clip_b.metadata['previous_metadata']['source_clip_name']
-    except KeyError:
-        pass
-
-    return False
-
-
 def read_from_string(input_str, rate=24, ignore_timecode_mismatch=False):
     """Reads a CMX Edit Decision List (EDL) from a string.
     Since EDLs don't contain metadata specifying the rate they are meant
@@ -915,7 +782,6 @@ def read_from_string(input_str, rate=24, ignore_timecode_mismatch=False):
         ignore_timecode_mismatch=ignore_timecode_mismatch
     )
     result = parser.timeline
-    result = _expand_transitions(result)
     return result
 
 
@@ -924,9 +790,9 @@ def write_to_string(input_otio, rate=None, style='avid', reelname_len=8):
     # also only works for a single video track at the moment
 
     video_tracks = [t for t in input_otio.tracks
-                    if t.kind == schema.TrackKind.Video]
+                    if t.kind == schema.TrackKind.Video and t.enabled]
     audio_tracks = [t for t in input_otio.tracks
-                    if t.kind == schema.TrackKind.Audio]
+                    if t.kind == schema.TrackKind.Audio and t.enabled]
 
     if len(video_tracks) != 1:
         raise exceptions.NotSupportedError(
@@ -1038,27 +904,30 @@ class EDLWriter(object):
                     )
                 )
             elif isinstance(child, schema.Clip):
-                events.append(
-                    Event(
-                        child,
-                        self._tracks,
-                        track.kind,
-                        self._rate,
-                        self._style,
-                        self._reelname_len
+                if child.enabled:
+                    events.append(
+                        Event(
+                            child,
+                            self._tracks,
+                            track.kind,
+                            self._rate,
+                            self._style,
+                            self._reelname_len
+                        )
                     )
-                )
+                else:
+                    pass
             elif isinstance(child, schema.Gap):
                 # Gaps are represented as missing record timecode, no event
                 # needed.
                 pass
 
         content = "TITLE: {}\n\n".format(title) if title else ''
-
-        # Convert each event/dissolve-event into plain text.
-        for idx, event in enumerate(events):
-            event.edit_number = idx + 1
-            content += event.to_edl_format() + '\n'
+        if track.enabled:
+            # Convert each event/dissolve-event into plain text.
+            for idx, event in enumerate(events):
+                event.edit_number = idx + 1
+                content += event.to_edl_format() + '\n'
 
         return content
 
