@@ -1,26 +1,5 @@
-#
+# SPDX-License-Identifier: Apache-2.0
 # Copyright Contributors to the OpenTimelineIO project
-#
-# Licensed under the Apache License, Version 2.0 (the "Apache License")
-# with the following modification; you may not use this file except in
-# compliance with the Apache License and the following modification to it:
-# Section 6. Trademarks. is deleted and replaced with:
-#
-# 6. Trademarks. This License does not grant permission to use the trade
-#    names, trademarks, service marks, or product names of the Licensor
-#    and its affiliates, except as required to comply with Section 4(c) of
-#    the License and to reproduce the content of the NOTICE file.
-#
-# You may obtain a copy of the Apache License at
-#
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the Apache License with the above modification is
-# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
-# KIND, either express or implied. See the Apache License for the specific
-# language governing permissions and limitations under the Apache License.
-#
 
 """OTIO Python Plugin Manifest system: locates plugins to OTIO."""
 
@@ -53,11 +32,12 @@ OTIO_PLUGIN_TYPES = [
     'schemadefs',
     'hook_scripts',
     'hooks',
+    'version_manifests',
 ]
 
 
 def manifest_from_file(filepath):
-    """Read the .json file at filepath into a Manifest object."""
+    """Read the .json file at filepath into a :py:class:`Manifest` object."""
 
     result = core.deserialize_json_from_file(filepath)
     absfilepath = os.path.abspath(filepath)
@@ -111,6 +91,8 @@ class Manifest(core.SerializableObject):
         self.hooks = {}
         self.hook_scripts = []
 
+        self.version_manifests = {}
+
     adapters = core.serializable_field(
         "adapters",
         type([]),
@@ -136,6 +118,11 @@ class Manifest(core.SerializableObject):
         type([]),
         "Scripts that can be attached to hooks."
     )
+    version_manifests = core.serializable_field(
+        "version_manifests",
+        type({}),
+        "Sets of versions to downgrade schemas to."
+    )
 
     def extend(self, another_manifest):
         """
@@ -152,7 +139,17 @@ class Manifest(core.SerializableObject):
         self.media_linkers.extend(another_manifest.media_linkers)
         self.hook_scripts.extend(another_manifest.hook_scripts)
 
+        for family, label_map in another_manifest.version_manifests.items():
+            # because self.version_manifests is an AnyDictionary instead of a
+            # vanilla python dictionary, it does not support the .set_default()
+            # method.
+            if family not in self.version_manifests:
+                self.version_manifests[family] = {}
+            self.version_manifests[family].update(label_map)
+
         for trigger_name, hooks in another_manifest.hooks.items():
+            # because self.hooks is an AnyDictionary instead of a vanilla
+            # python dictionary, it does not support the .set_default() method.
             if trigger_name not in self.hooks:
                 self.hooks[trigger_name] = []
             self.hooks[trigger_name].extend(hooks)
@@ -219,22 +216,24 @@ _MANIFEST = None
 
 
 def load_manifest():
-    """ Walk the plugin manifest discovery systems and accumulate manifests.
+    """Walk the plugin manifest discovery systems and accumulate manifests.
 
     The order of loading (and precedence) is:
-        1. manifests specfied via the OTIO_PLUGIN_MANIFEST_PATH variable
-        2. builtin plugin manifest
-        3. contrib plugin manifest
-        4. setuptools.pkg_resources based plugin manifests
+
+       1. Manifests specified via the ``OTIO_PLUGIN_MANIFEST_PATH`` variable
+       2. Entrypoint based plugin manifests
+       3. Builtin plugin manifest
+       4. Contrib plugin manifest
     """
 
     result = Manifest()
 
     # Read plugin manifests defined on the $OTIO_PLUGIN_MANIFEST_PATH
     # environment variable.  This variable is an os.pathsep separated list of
-    # file paths to manifest json files.
+    # file paths to manifest json files. Can be set to "" to indicate no
+    # local custom manifest path.
     _local_manifest_path = os.environ.get("OTIO_PLUGIN_MANIFEST_PATH", None)
-    if _local_manifest_path is not None:
+    if _local_manifest_path:
         for src_json_path in _local_manifest_path.split(os.pathsep):
             json_path = os.path.abspath(src_json_path)
             if (
@@ -251,32 +250,6 @@ def load_manifest():
 
             result.extend(manifest_from_file(json_path))
 
-    # the builtin plugin manifest
-    builtin_manifest_path = os.path.join(
-        os.path.dirname(os.path.dirname(inspect.getsourcefile(core))),
-        "adapters",
-        "builtin_adapters.plugin_manifest.json"
-    )
-    if os.path.abspath(builtin_manifest_path) not in result.source_files:
-        plugin_manifest = manifest_from_file(builtin_manifest_path)
-        result.extend(plugin_manifest)
-
-    # the contrib plugin manifest (located in the opentimelineio_contrib package)
-    try:
-        import opentimelineio_contrib as otio_c
-
-        contrib_manifest_path = os.path.join(
-            os.path.dirname(inspect.getsourcefile(otio_c)),
-            "adapters",
-            "contrib_adapters.plugin_manifest.json"
-        )
-        if os.path.abspath(contrib_manifest_path) not in result.source_files:
-            contrib_manifest = manifest_from_file(contrib_manifest_path)
-            result.extend(contrib_manifest)
-
-    except ImportError:
-        pass
-
     # setuptools.pkg_resources based plugins
     if pkg_resources:
         for plugin in pkg_resources.iter_entry_points(
@@ -343,6 +316,32 @@ def load_manifest():
         #        available?
         pass
 
+    # the builtin plugin manifest
+    builtin_manifest_path = os.path.join(
+        os.path.dirname(os.path.dirname(inspect.getsourcefile(core))),
+        "adapters",
+        "builtin_adapters.plugin_manifest.json"
+    )
+    if os.path.abspath(builtin_manifest_path) not in result.source_files:
+        plugin_manifest = manifest_from_file(builtin_manifest_path)
+        result.extend(plugin_manifest)
+
+    # the contrib plugin manifest (located in the opentimelineio_contrib package)
+    try:
+        import opentimelineio_contrib as otio_c
+
+        contrib_manifest_path = os.path.join(
+            os.path.dirname(inspect.getsourcefile(otio_c)),
+            "adapters",
+            "contrib_adapters.plugin_manifest.json"
+        )
+        if os.path.abspath(contrib_manifest_path) not in result.source_files:
+            contrib_manifest = manifest_from_file(contrib_manifest_path)
+            result.extend(contrib_manifest)
+
+    except ImportError:
+        pass
+
     # force the schemadefs to load and add to schemadef module namespace
     for s in result.schemadefs:
         s.module()
